'use-strict'
// -- // Our lovely functions // -- //
const SHEET = document.getElementById('sheet');
const FOOTER = document.getElementsByTagName('footer')[0];
const COLOR_BAR = document.getElementById('color_bar');
const GALLERY = document.getElementById('gallery');

// Building Color Bar
colorBar = function (colors) {
    colors.forEach((item, i) => {
        let bar = document.createElement('div');
        bar.style.setProperty('background', item);
        COLOR_BAR.appendChild(bar);
    });

};

// Building Top Section
topSection = function (top) {
    let top_sec = document.createElement('section');
    top_sec.setAttribute('id', 'top');

    // Create Left Div
    let left_side = document.createElement('div');

    let name = document.createElement('h1');
    name.innerHTML = top.name;
    left_side.appendChild(name);

    for (x in top.bio) {
        let line = document.createElement('p');
        line.innerHTML = `<b>${x}:</b> ${top.bio[x]}`;
        left_side.appendChild(line);
    }

    let quote = document.createElement('blockquote');
    quote.innerHTML = top.quote;
    left_side.appendChild(quote);

    // Create Character Image
    let image = document.createElement('div');
    let frame = document.createElement('img');

    frame.src = top.frame;
    frame.setAttribute('draggable', false);

    image.className = 'character_img';
    image.style.backgroundImage = `url('${IMG_LOCATION}${top.image}')`;
    image.appendChild(frame);

    // Add to Page
    top_sec.appendChild(left_side);
    top_sec.appendChild(image);
    SHEET.appendChild(top_sec);
}

// Building all the Middle Sections
middleSections = function () {
    MIDDLE.forEach((item, i) => {
        if (item.type == 1) {
            let sec = document.createElement('section');
            let body = document.getElementById(item.content).content.cloneNode(true);

            if (item.title != null) {
                let header = document.createElement('h2');
                header.innerHTML = item.title;
                sec.appendChild(header);
            }

            sec.appendChild(body);
            SHEET.appendChild(sec);
        }
        else if (item.type == 2) {
            let double = document.createElement('div');
            double.className = "double";
            for (let i = 0; i < 2; i++) {
                let sec = document.createElement('section');
                let body = document.getElementById(item.content[i]).content.cloneNode(true);

                if (item.title[i] != null) {
                    let header = document.createElement('h2');
                    header.innerHTML = item.title[i];
                    sec.appendChild(header);
                }

                sec.appendChild(body);
                double.appendChild(sec);
            }
            SHEET.appendChild(double);
        }
        else if (item.type == 3) {
            let sec = document.createElement('section');
            let header = document.createElement('h2');
            let gallery = gallerySection(item.content);

            header.innerHTML = item.title;

            sec.appendChild(header);
            sec.appendChild(gallery);
            SHEET.appendChild(sec);
        }
        else {
            alert(`Error loading section ${i + 1}!`);
            console.log(`Error loading section ${i + 1}!`);
        }
    });

}

// Building Gallery
let galleries = [];
let currentImage = [0,0];

gallerySection = function (list) {
    let gallery = document.createElement('div');
    gallery.className = "gallery";

    list.forEach((item, i) => {
        let pic = document.createElement('div');
        pic.style.backgroundImage = `url('${IMG_LOCATION}${item[1]}')`;

        let overlay = document.createElement('div');
        overlay.className = 'overlay';
        pic.appendChild(overlay);

        let index = galleries.length
        pic.addEventListener('click', function(){
            openModal([index, i]);
            currentImage[0] = index;
            currentImage[1] = i;
        })

        gallery.appendChild(pic);
    });

    galleries.push(list);
    return gallery;
}

// Changing modal image
prevModal = function () {
    if (currentImage[1] != 0) {
        currentImage[1] -= 1;
        openModal(currentImage);
    }
}
document.getElementsByName('prev')[0].addEventListener('click', prevModal);

nextModal = function () {
    if (currentImage[1] != (galleries[currentImage[0]].length - 1)) {
        currentImage[1] += 1;
        openModal(currentImage);
    }
}
document.getElementsByName('next')[0].addEventListener('click', nextModal);

// Displaying Gallery Modal
openModal = function (image) {
    let img = document.getElementById('gal_img');
    let title = document.getElementById('gal_title');
    let desc = document.getElementById('gal_desc');

    title.innerHTML = galleries[image[0]][image[1]][0];
    img.style.backgroundImage = `url('${IMG_LOCATION}${galleries[image[0]][image[1]][1]}')`;
    desc.innerHTML = galleries[image[0]][image[1]][2];

    GALLERY.style.setProperty('display', 'grid');
}

closeModal = function () {
    GALLERY.style.setProperty('display', 'none');
}
document.getElementsByName('close')[0].addEventListener('click', closeModal);

// -- Building the Page -- //
console.log('Built using RefMine!')

colorBar(TOP.colors);
topSection(TOP);
middleSections();

console.log('Building done. ✌🏿')
